<?php

namespace Drupal\Tests\file_extractor\Kernel\Extractor;

use Drupal\Core\File\FileSystemInterface;
use Drupal\file\FileInterface;
use Drupal\file_extractor\Extractor\ExtractorPluginBase;
use Drupal\KernelTests\KernelTestBase;

/**
 * Base class for extractor plugins kernel tests.
 */
abstract class ExtractorTestBase extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'file',
    'file_extractor',
    'system',
    'user',
  ];

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * File system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The tested extractor plugin ID.
   *
   * @var string
   */
  protected $extractorPluginId;

  /**
   * The tested extractor plugin configuration.
   *
   * @var array
   */
  protected $extractorPluginConfiguration = [];

  /**
   * The tested extractor.
   *
   * @var \Drupal\file_extractor\Extractor\ExtractorPluginInterface
   */
  protected $extractor;

  /**
   * The test file.
   *
   * @var \Drupal\file\FileInterface
   */
  protected $testFile;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $this->entityTypeManager = $this->container->get('entity_type.manager');
    $this->fileSystem = $this->container->get('file_system');

    /** @var \Drupal\file_extractor\Extractor\ExtractorPluginManager $extractor_plugin_manager */
    $extractor_plugin_manager = $this->container->get('plugin.manager.file_extractor.extractor');
    $this->extractor = $extractor_plugin_manager->createInstance($this->extractorPluginId, $this->extractorPluginConfiguration);
  }

  /**
   * Helper function to get the test file.
   *
   * @return \Drupal\file\FileInterface
   *   The file used for the test.
   */
  protected function getTestFile() : FileInterface {
    if (!isset($this->testFile)) {
      // Copy the source file to public directory.
      $source = drupal_get_path('module', 'file_extractor');
      $source .= '/data/' . ExtractorPluginBase::TEST_FILENAME;
      $this->fileSystem->copy($source, ExtractorPluginBase::TEST_FILE_URI, FileSystemInterface::EXISTS_REPLACE);

      /** @var \Drupal\file\FileInterface $file */
      $this->testFile = $this->entityTypeManager->getStorage('file')->create([
        'filename' => ExtractorPluginBase::TEST_FILENAME,
        'filemime' => 'application/pdf',
        'uri' => ExtractorPluginBase::TEST_FILE_URI,
      ]);
    }

    return $this->testFile;
  }

  /**
   * Tests extraction method.
   */
  public function testExtraction() {
    $extraction = $this->extractor->extract($this->getTestFile());
    $this->assertNotEmpty($extraction, 'The extraction with the plugin ' . $this->extractorPluginId . ' is not working.');
  }

}
